package edu.lawrence.mazedrawing;

import java.io.File;
import java.io.PrintWriter;
import java.util.ArrayList;
import javafx.scene.input.KeyEvent;
import javafx.scene.input.MouseEvent;
import javafx.scene.layout.Pane;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.scene.shape.Line;
import javafx.scene.shape.Shape;

public class MazePane extends Pane {

    private ArrayList<Wall> walls;
    private ArrayList<Treasure> treasure;
    private Wall dragging;
    private Treasure placing;
    private Color color;
    private boolean drawingWalls;

    public MazePane() {
        drawingWalls = true;
        color = Color.RED;
        walls = new ArrayList<Wall>();
        treasure = new ArrayList<Treasure>();
        this.setOnMousePressed(e -> startDrag(e));
        this.setOnMouseDragged(e -> dragSegment(e));
        this.setOnMouseReleased(e -> endDrag(e));
        this.setOnKeyPressed(e -> keyPress(e));
        for (int row = 0; row <= 11; row++) {
            for (int col = 0; col <= 16; col++) {
                Circle circle = new Circle();
                circle.setCenterX(col * 40);
                circle.setCenterY(row * 40);
                circle.setFill(Color.RED);
                circle.setRadius(2);
                this.getChildren().add(circle);
            }
        }
    }

    public void startDrag(MouseEvent e) {
        if (drawingWalls) {
            dragging = new Wall(e.getX(), e.getY());
            walls.add(dragging);
            this.getChildren().add(dragging.getShape());
            placing = null;
        } else {
            placing = new Treasure(e.getX(), e.getY(), color);
            treasure.add(placing);
            this.getChildren().add(placing.getShape());
            dragging = null;
        }
    }

    public void dragSegment(MouseEvent e) {
        if (dragging != null) {
            dragging.setEnd(e.getX(), e.getY());
        } else if (placing != null) {
            placing.setLocation(e.getX(), e.getY());
        }
    }

    public void endDrag(MouseEvent e) {
        if (dragging != null) {
            dragging.snapToGrid(e.getX(), e.getY());
            /** Sample code to check for line intersections.
            Line l = (Line) dragging.getShape();
            for(Wall w : walls) {
                if(w != dragging) {
                    Shape intersection = Shape.intersect(w.getShape(), l);
                    if(intersection.getBoundsInLocal().getWidth() > 0)
                        l.setStroke(Color.RED);
                }
            } **/
            dragging = null;
        }
        if (placing != null) {
            placing.snapToGrid(e.getX(), e.getY());
            placing = null;
        }

    }

    public void keyPress(KeyEvent e) {
        switch (e.getCode()) {
            case R:
                color = Color.RED;
                break;
            case G:
                color = Color.GREEN;
                break;
            case B:
                color = Color.BLUE;
                break;
            case BACK_SPACE:
                Wall toRemove = walls.get(walls.size() - 1);
                walls.remove(toRemove);
                this.getChildren().remove(toRemove.getShape());
                break;
            case P:
                drawingWalls = !drawingWalls;
                break;
        }
    }

    public void save() {
        try {
            PrintWriter out = new PrintWriter(new File("maze.txt"));
            out.println(walls.size());
            for (Wall w : walls) {
                w.write(out);
            }
            out.println(treasure.size());
            for (Treasure g : treasure) {
                g.write(out);
            }
            out.close();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}
